from werkzeug.security import generate_password_hash, check_password_hash
from flask_login import UserMixin
from datetime import datetime
from mpole import db, login

class Upload(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    assessment_title = db.Column(db.String(50), nullable=False)
    filename = db.Column(db.String(50), nullable=False)
    data = db.Column(db.LargeBinary(length=(2**32)-1))
    deadline = db.Column(db.DateTime, index=True, nullable=False, default=datetime.utcnow)
    user_id = db.Column(db.Integer, db.ForeignKey('user.id'), nullable=False)

class User(UserMixin, db.Model):
    id = db.Column(db.Integer, primary_key=True)
    first_name = db.Column(db.String(20), nullable=False)
    last_name = db.Column(db.String(30), nullable=False)
    email = db.Column(db.String(120), index=True, nullable=False, unique=True)
    password_hash = db.Column(db.String(128))
    posts = db.relationship('Post', backref='author', lazy='dynamic')
    uploads = db.relationship('Upload', backref='uploader', lazy='dynamic')

    def __repr__(self):
        return f"User('{self.first_name}', '{self.last_name}' ,'{self.email}')"

    def set_password(self, password):
        self.password_hash = generate_password_hash(password)

    def verify_password(self, password):
        return check_password_hash(self.password_hash, password)

class Post(db.Model):
    id = db.Column(db.Integer, primary_key = True)
    body = db.Column(db.String(140), nullable=False)
    timestamp = db.Column(db.DateTime, index=True, nullable=False, default=datetime.utcnow)
    user_id = db.Column(db.Integer, db.ForeignKey('user.id'), nullable=False)

    def __repr__(self):
        return f"Post('{self.body}')"

class Student(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    first_name = db.Column(db.String(20), nullable=False)
    last_name = db.Column(db.String(30), nullable=False)
    email = db.Column(db.String(120), index=True, nullable=False, unique=True)
    password_hash = db.Column(db.String(128))
    posts = db.relationship('StudentPost', backref='author', lazy='dynamic')

    def __repr__(self):
        return f"User('{self.first_name}', '{self.last_name}' ,'{self.email}')"

class StudentPost(db.Model):
    id = db.Column(db.Integer, primary_key = True)
    body = db.Column(db.String(140), nullable=False)
    timestamp = db.Column(db.DateTime, index=True, nullable=False, default=datetime.utcnow)
    user_id = db.Column(db.Integer, db.ForeignKey('student.id'), nullable=False)

    def __repr__(self):
        return f"Post('{self.body}')"

@login.user_loader
def load_user(id):
    return User.query.get(int(id))
