import os
import cv2
import numpy as np
from io import BytesIO
from datetime import datetime
from flask import render_template, url_for, request, redirect, flash, session, send_file, Response
from werkzeug.urls import url_parse
from mpole import app, db
from flask_login import current_user, login_user, logout_user, login_required
from mpole.models import User, Post, Upload
from mpole.forms import LoginForm, RegistrationForm, PostForm, UploadForm


@app.route("/")
def index():
    return redirect(url_for('login'))

@app.route("/login", methods=['GET', 'POST'])
def login():
    if current_user.is_authenticated:
        return redirect(url_for('home'))
    form = LoginForm()
    if form.validate_on_submit():
        user = User.query.filter_by(email=form.email.data).first()
        if user is None or not user.verify_password(form.password.data):
            flash('Invalid email or password')
            return redirect(url_for('login'))
        login_user(user, remember=form.remember_me.data)
        next_page = request.args.get('next')
        if not next_page or url_parse(next_page).netloc !='':
            next_page = url_for('home')
        return redirect(next_page)
    return render_template('login.html', title='Log In', form=form)

@app.route("/logout")
def logout():
    logout_user()
    return redirect(url_for('login'))

@app.route("/register", methods=['GET', 'POST'])
def register():
    if current_user.is_authenticated:
        return redirect(url_for('home'))
    form = RegistrationForm()
    if form.validate_on_submit():
        user = User(first_name=form.first_name.data, last_name=form.last_name.data, email=form.email.data)
        user.set_password(form.password.data)
        db.session.add(user)
        db.session.commit()
        flash('Registraion Successful!')
        return redirect(url_for('login'))
    return render_template('register.html', title='Register', form=form)

@app.route("/home", methods=['GET', 'POST'])
@login_required
def home():
    form = PostForm()
    if form.validate_on_submit():
        post = Post(body=form.post.data, author=current_user)
        db.session.add(post)
        db.session.commit()
        flash('Your post has been shared!')
        return redirect(url_for('home'))
    posts = Post.query.order_by(Post.timestamp.desc()).limit(3).all()
    queries = [
        {
            'author' : {'username': 'Alice'},
            'body': 'Can you upload the explanation for Question 4?'
        },
        {
            'author' : {'username': 'Emily'},
            'body' : 'Do you have any more question papers?'
        },
        {
            'author' : {'username': 'Charlie'},
            'body' : 'Can you extend the deadline for Assssment 1 please?'
        }
    ]
    return render_template('home.html', title='Home', posts=posts, queries=queries, form=form)

@app.route("/page1", methods=['GET', 'POST'])
@login_required
def page1():
    form = PostForm()
    if form.submit.data and form.validate_on_submit():
        post = Post(body=form.post.data, author=current_user)
        db.session.add(post)
        db.session.commit()
        flash('Your post has been shared!')
        return redirect(url_for('page1'))
    form2 = UploadForm()
    if form2.submit2.data and form2.validate_on_submit():
        file = form2.data.data
        assessment_title = form2.assessment_title.data
        filename = file.filename
        data = file.read()
        deadline = datetime.strptime(str(form2.deadline.data), '%Y-%m-%d %H:%M:%S')

        upload = Upload(assessment_title=assessment_title, filename=filename, data=data, deadline=deadline, uploader=current_user)
        db.session.add(upload)
        db.session.commit()
        flash('Your Assessment has been created!')
        return redirect(url_for('page1'))
    queries = [
        {
            'author' : {'username': 'Alice'},
            'body': 'Can you upload the explanation for Question 4?'
        },
        {
            'author' : {'username': 'Emily'},
            'body' : 'Do you have any more question papers?'
        },
        {
            'author' : {'username': 'Charlie'},
            'body' : 'Can you extend the deadline for Assssment 1 please?'
        }
    ]
    posts = Post.query.order_by(Post.timestamp.desc()).limit(3).all()
    uploads = Upload.query.filter_by(user_id=current_user.id).order_by(Upload.deadline.asc()).all()
    return render_template('page1.html', title='Page 1', posts=posts, queries=queries, form=form, form2=form2, uploads=uploads)

@app.route('/download/<upload_id>')
def download(upload_id):
    upload = Upload.query.filter_by(id=upload_id).first()
    return send_file(BytesIO(upload.data), attachment_filename=upload.filename, as_attachment=True)
    return redirect(url_for('page1'))

@app.route("/page2", methods=['GET', 'POST'])
@login_required
def page2():
    form = PostForm()
    if form.validate_on_submit():
        post = Post(body=form.post.data, author=current_user)
        db.session.add(post)
        db.session.commit()
        flash('Your post has been shared!')
        return redirect(url_for('page2'))
    posts = Post.query.order_by(Post.timestamp.desc()).limit(3).all()
    queries = [
        {
            'author' : {'username': 'Alice'},
            'body': 'Can you upload the explanation for Question 4?'
        },
        {
            'author' : {'username': 'Emily'},
            'body' : 'Do you have any more question papers?'
        },
        {
            'author' : {'username': 'Charlie'},
            'body' : 'Can you extend the deadline for Assssment 1 please?'
        }
    ]
    return render_template('page2.html', title='Page 2', posts=posts, queries=queries, form=form)

camera=cv2.VideoCapture(0)

def create_frames():
    while True:
        success,frame=camera.read()
        retval,buffer=cv2.imencode('.jpg', frame)
        frame=buffer.tobytes()
        yield(b'--frame\r\n'
                b'Content-Type: image/jpeg\r\n\r\n' + frame + b'\r\n')

        if cv2.waitKey(0) == ord('w'):
            break

    camera.release()
    cv2.destroyAllWindows()

@app.route("/video")
def video():
    return Response(create_frames(), mimetype='multipart/x-mixed-replace; boundary=frame')



@app.route("/page3", methods=['GET', 'POST'])
@login_required
def page3():
    form = PostForm()
    if form.validate_on_submit():
        post = Post(body=form.post.data, author=current_user)
        db.session.add(post)
        db.session.commit()
        flash('Your post has been shared!')
        return redirect(url_for('page3'))
    posts = Post.query.order_by(Post.timestamp.desc()).limit(3).all()
    queries = [
        {
            'author' : {'username': 'Alice'},
            'body': 'Can you upload the explanation for Question 4?'
        },
        {
            'author' : {'username': 'Emily'},
            'body' : 'Do you have any more question papers?'
        },
        {
            'author' : {'username': 'Charlie'},
            'body' : 'Can you extend the deadline for Assssment 1 please?'
        }
    ]
    return render_template('page3.html', title='Page 3', posts=posts, queries=queries, form=form)

@app.route("/page4", methods=['GET', 'POST'])
@login_required
def page4():
    form = PostForm()
    if form.validate_on_submit():
        post = Post(body=form.post.data, author=current_user)
        db.session.add(post)
        db.session.commit()
        flash('Your post has been shared!')
        return redirect(url_for('page4'))
    posts = Post.query.order_by(Post.timestamp.desc()).limit(3).all()
    queries = [
        {
            'author' : {'username': 'Alice'},
            'body': 'Can you upload the explanation for Question 4?'
        },
        {
            'author' : {'username': 'Emily'},
            'body' : 'Do you have any more question papers?'
        },
        {
            'author' : {'username': 'Charlie'},
            'body' : 'Can you extend the deadline for Assssment 1 please?'
        }
    ]
    return render_template('page4.html', title='Page 4', posts=posts, queries=queries, form=form)

@app.route("/page5", methods=['GET', 'POST'])
@login_required
def page5():
    form = PostForm()
    if form.validate_on_submit():
        post = Post(body=form.post.data, author=current_user)
        db.session.add(post)
        db.session.commit()
        flash('Your post has been shared!')
        return redirect(url_for('page5'))
    posts = Post.query.order_by(Post.timestamp.desc()).limit(3).all()
    queries = [
        {
            'author' : {'username': 'Alice'},
            'body': 'Can you upload the explanation for Question 4?'
        },
        {
            'author' : {'username': 'Emily'},
            'body' : 'Do you have any more question papers?'
        },
        {
            'author' : {'username': 'Charlie'},
            'body' : 'Can you extend the deadline for Assssment 1 please?'
        }
    ]
    return render_template('page5.html', title='Page 5', posts=posts, queries=queries, form=form)
